#ifndef INC_FILTER
#define INC_FILTER

/**************************************************************
*
*                FILTER.HPP
*
**************************************************************/

typedef enum {off, on} SWITCH ;

/*** channel filter values ***/
#define all_channels_on   0xffff  
#define all_channels_off  0x0000 

#include <notimtab.h>

#define max_track_name_length 29

typedef struct 
{
	char name[max_track_name_length] ;
	NOTE_TIME_TABLE_ELEMENT *first_element ;
	__int32 elements_generated ;
	NOTE min_note, max_note ;
	SWITCH track_filter ; 
	unsigned __int16 channel_filter, 
	                 channels ;
	int last ;   /* TRUE, if last element in table */
} 
TRACK_TABLE_ELEMENT ;


# if defined (__PUREC__)
/******** init data for filter constructor *******/

typedef struct 
{
	CICONBLK *icon ;
	char *filename ;              /* for icon text, info line */
	unsigned char *RAM_file ;     /* pointer to MIDI file in RAM */
	long filesize ;               /* size of MIDI file */
	int ntrks ;                   /* number of tracks in MIDI file */
	SWITCH track_filter ;         /* init value track filters */
	unsigned int channel_filter ; /* init value channel filters */
} FILTER_INIT ;

/*******************************************************************
*/
	Awindow *FILTER_constructor(FILTER_INIT *) ;
/*
* Constructor of the FILTER object. It
* constructs an object instance of the FILTER object containing
* ACS object and own data instances.
* 
* OUTPUT:  filter icon placed on screen and
*          own FILTER data instance connected to ACS-window
*          instance
* RETURN:  errors: NULL or -if successfull- ACS-window pointer,
*          which will be used as handle for the object instance
* 
*******************************************************************/


/*******************************************************************
*/
	void FILTER_destructor(Awindow *) ;
/*
* Destructor of the FILTER object. It
* frees all memory used by ACS and own data instances.
* 
*******************************************************************/


/*******************************************************************
*/
	long get_transformed_size(Awindow *window) ;
/*
*   Delivers the total size of the 
*   transformed MIDI file ( = size of all note time tables together).
* 
*******************************************************************/
#endif

/*******************************************************************
*/
	void get_absolute_min_max_note(Awindow *window, 
                               NOTE *min_note, NOTE *max_note) ;
/*
* Determines the minimum note and 
* the maximum note of all note time tables for which the track 
* switch is set to "on". If all tracks are switched off minimum and
* maximum note are set to 0.
* 
* OUTPUT:  *min_note: absolute minimum note of all enabled tracks
*          *max_note: absolute maximum note of all enabled tracks
* 
*******************************************************************/


/*******************************************************************
*/
	long get_absolute_max_time(Awindow *window) ;
/*
* Delivers the total maximum time of all 
* last elements of all note time tables for which the track switch
* is set to "on". If all tracks are switched off 0 is returned. 
* 
*******************************************************************/


/*******************************************************************
*/
	int get_first_active_track(Awindow *window) ;
/*
* Delivers the lowest track number of all  
* track numbers of enabled (switched to "on") tracks. 
* 
* RETURN:  track number of first enabled track,
*          -1 if no track is enabled 
* 
*******************************************************************/


/*******************************************************************
*/
	int get_last_active_track(Awindow *window) ;
/*
* Delivers the highest track number of all  
* track numbers of enabled (switched to "on") tracks. 
* 
* RETURN:  track number of last enabled track,
*          -1 if no track is enabled 
* 
*******************************************************************/


/*******************************************************************
*/
	char *get_track_name(Awindow *window, int track) ;
/*
* "get_track_name" delivers the (MIDI file) track name of the   
* desired track.
* 
* INPUT:   window:    pointer to current window instance
*          track:     for this track the name is inquired
* RETURN:  pointer to track name 
* 
*******************************************************************/


/*******************************************************************
*/
	int get_min_max(Awindow *window, int track, 
				TIME min_time , TIME max_time, 
            	NOTE *min_note, NOTE *max_note) ;
/*
* Searches between min_time and max_time for the 
* lowest and the highest note inside a note time table. 
* Only tracks and channels that are switched on in the filter table
* are taken into account.
* 
* INPUT:   first_n:             pointer to first 
*                               note time table element
*          elements_generated:  number of elements in note time table
*          first_f:             pointer to first filter table element
*          min_time, max_time:  limits inside which to search for 
*                               minium and maximum note
* OUTPUT:  *P_min_note:         minimum note found 
*          *P_max_note:         maximum note found
* RETURN:  0 or 1:  if track is disabled or enabled, respectively
* 
*******************************************************************/	


/*******************************************************************
*/
	void draw_track
	(
		int handle,      /* in: VDI handle for drawing */
		Awindow *window, /* in: affected ACS window instance */
		int track,       /* in: number of track to draw */
		int x_min, int x_max, /* drawing to be spread between x_min and x_max */
		int y,           /* y value where to draw note_of_y */
		float dy,        /* distance between succeeding notes */
		TIME min_time, TIME max_time, /* the notes between these times shall be spread over x_min/x_max */
		NOTE note_of_y,  /* note which corresponds to y */
		float average_height, /* of a note */
		float dyn_scale,      /* < 1, sensitivity of note height from note dynamic */
		int type,             /* note type: with/without head/tail, ... (mask) */
		int style[12],        /* VDI fill style for note bodies */
		int color[12],        /* VDI color for note bodies */
		int end_style[12],    /* VDI fill style for note ends */
		int end_color[12]     /* VDI color for note ends */
	) ;
/*
* draws all notes of the specified track which 
* are located between "min_time" and "max_time". For drawing
* VDI routines are used with "handle" as parameter.
* 
*******************************************************************/

#endif /* include blocker */